from pydantic import AliasChoices, AnyUrl, Field, HttpUrl, SecretStr
from pydantic_settings import BaseSettings

from haidra_core.meta import default_settings_config

DEFAULT_AI_HORDE_URL = HttpUrl("https://aihorde.net/api/")
DEFAULT_ALT_AI_HORDE_URLS: list[AnyUrl] = [HttpUrl("https://stablehorde.net/api/")]
DEFAULT_RATINGS_URL = HttpUrl("https://ratings.aihorde.net/api/")

DEFAULT_ANONYMOUS_API_KEY = "0000000000"

DEFAULT_LOGS_FOLDER = "./logs"
DEFAULT_MODELS_FOLDER = "./models"


class AIHordeSettings(BaseSettings):
    """Base settings for AI Horde configurations."""

    model_config = default_settings_config

    ai_horde_url: AnyUrl = Field(
        default=DEFAULT_AI_HORDE_URL,
        validation_alias=AliasChoices("HORDE_URL", "AI_HORDE_URL"),
    )
    """The URL for this AI Horde instance. If more than one, additional URLs are in the field `alt_horde_urls`."""

    alt_horde_urls: list[AnyUrl] = Field(default=DEFAULT_ALT_AI_HORDE_URLS)
    """Alternative API endpoints for the AI Horde. These should all lead to the same logical AI Horde."""


class AIHordeServerSettings(AIHordeSettings):
    """Base settings for AI Horde server configurations."""


class AIHordeClientSettings(AIHordeSettings):
    """Base settings for an AI Horde client."""

    api_key: SecretStr = Field(default=SecretStr(DEFAULT_ANONYMOUS_API_KEY))
    """The API key used for authenticating requests to the AI Horde."""

    ratings_url: AnyUrl = Field(default=DEFAULT_RATINGS_URL)
    """The API endpoint for AI Horde ratings."""

    logs_folder: str = DEFAULT_LOGS_FOLDER
    """The folder where application logs are stored."""


class AIHordeWorkerSettings(AIHordeClientSettings):
    """Settings for an AI Horde worker."""

    aiworker_cache_home: str = DEFAULT_MODELS_FOLDER
    """The folder where AI worker (or client) files are stored, most notably models and checkpoints."""



from pydantic import SecretStr

from haidra_core.ai_horde import AIHordeClientSettings, AIHordeWorkerSettings
from haidra_core.ai_horde.settings import AIHordeServerSettings


def test_ai_horde_client_settings() -> None:
    """Verify AIHordeClientSettings with default values."""
    settings = AIHordeClientSettings(api_key=SecretStr("test_api_key"))
    assert settings.api_key.get_secret_value() == "test_api_key"
    assert settings.ai_horde_url.encoded_string() == "https://aihorde.net/api/"
    assert settings.alt_horde_urls[0].encoded_string() == "https://stablehorde.net/api/"
    assert settings.ratings_url.encoded_string() == "https://ratings.aihorde.net/api/"
    assert settings.logs_folder == "./logs"


def test_ai_horde_worker_settings() -> None:
    """Verify AIHordeWorkerSettings with custom values."""
    settings = AIHordeWorkerSettings(
        api_key=SecretStr("test_worker_api_key"),
        aiworker_cache_home="./__worker_models",
    )
    assert settings.api_key.get_secret_value() == "test_worker_api_key"
    assert settings.ai_horde_url.encoded_string() == "https://aihorde.net/api/"
    assert settings.alt_horde_urls[0].encoded_string() == "https://stablehorde.net/api/"
    assert settings.ratings_url.encoded_string() == "https://ratings.aihorde.net/api/"
    assert settings.logs_folder == "./logs"
    assert settings.aiworker_cache_home == "./__worker_models"


def test_ai_horde_server_settings() -> None:
    """Verify AIHordeServerSettings with default values."""
    import os

    original_horde_url = os.environ.get("HORDE_URL")
    original_ai_horde_url = os.environ.get("AI_HORDE_URL")

    os.environ["HORDE_URL"] = "https://test.local/api/"

    if original_ai_horde_url is not None:
        del os.environ["AI_HORDE_URL"]

    settings = AIHordeServerSettings()
    assert settings.ai_horde_url.encoded_string() == "https://test.local/api/"

    del os.environ["HORDE_URL"]

    os.environ["AI_HORDE_URL"] = "https://test2.local/api/"
    settings = AIHordeServerSettings()
    assert settings.ai_horde_url.encoded_string() == "https://test2.local/api/"

    del os.environ["AI_HORDE_URL"]

    if original_horde_url is not None:
        os.environ["HORDE_URL"] = original_horde_url

    if original_ai_horde_url is not None:
        os.environ["AI_HORDE_URL"] = original_ai_horde_url


def test_settings_by_environment_variable() -> None:
    """Test settings can be overridden by environment variables."""
    import os

    from haidra_core.ai_horde.settings import AIHordeWorkerSettings

    aiworker_cache_home = os.environ.get("AIWORKER_CACHE_HOME")
    original_cache_home = aiworker_cache_home

    if aiworker_cache_home is None:
        aiworker_cache_home = "./__worker_models"
        os.environ["AIWORKER_CACHE_HOME"] = aiworker_cache_home

    settings = AIHordeWorkerSettings()

    assert settings.aiworker_cache_home == aiworker_cache_home

    if original_cache_home is None:
        del os.environ["AIWORKER_CACHE_HOME"]
    else:
        os.environ["AIWORKER_CACHE_HOME"] = original_cache_home



from pydantic import Field

from haidra_core.meta import SharedCISettings

from .settings import DEFAULT_MODELS_FOLDER


class AIHordeCISettings(SharedCISettings):
    """Settings for AI Horde CI/CD pipelines."""

    hordelib_ci_ongoing: bool = Field(default=False)
    """Indicates if the hordelib CI/CD pipeline is ongoing."""

    horde_sdk_testing: bool = Field(default=False)
    """Indicates if the AI Horde SDK is currently being tested."""

    ai_horde_testing: bool = Field(default=False)
    """Indicates if the AI Horde is currently being tested."""

    aiworker_cache_home: str = DEFAULT_MODELS_FOLDER
    """The folder where AI worker models are stored."""



from haidra_core.service_base import ContainsMessage, ContainsMessageReturnCode, ContainsReturnCode, ContainsStatus


def test_fields_as_expected() -> None:
    """Test that the fields of the models are as expected."""
    ContainsStatus.model_validate({"status": "ok"})
    ContainsMessage.model_validate({"message": "Hello, world!"})
    crc = ContainsReturnCode.model_validate({"rc": 0})
    assert hasattr(crc, "return_code"), "`return_code` is more readable than `rc`."

    message_and_return_code = ContainsMessageReturnCode.model_validate({"message": "Hello, world!", "rc": 0})
    assert isinstance(message_and_return_code, ContainsMessageReturnCode)
    assert isinstance(message_and_return_code, ContainsMessage)
    assert hasattr(message_and_return_code, "message")
    assert hasattr(message_and_return_code, "return_code")



"""Contains AI-Horde specific classes which are used across the ecosystem."""

from .settings import AIHordeClientSettings, AIHordeServerSettings, AIHordeWorkerSettings

__all__ = [
    "AIHordeClientSettings",
    "AIHordeServerSettings",
    "AIHordeWorkerSettings",
]



from pydantic import AliasChoices, BaseModel, Field


class ContainsMessage(BaseModel):
    """A model that contains a message field."""

    message: str


class ContainsStatus(BaseModel):
    """A model that contains a status field."""

    status: str


class ContainsReturnCode(BaseModel):
    """A model that contains a return code field."""

    return_code: int = Field(
        validation_alias=AliasChoices(
            "return_code",
            "rc",
        ),
    )


class ContainsMessageReturnCode(ContainsMessage, ContainsReturnCode):
    """A model that contains both a message and a return code."""



"""Information about and for meta-processes, such as CI/CD, monitoring, and logging."""

import os

from pydantic import Field
from pydantic_settings import BaseSettings, SettingsConfigDict

from haidra_core import default_env_files

use_default_env_files = os.environ.get("HAIDRA_USE_DEFAULT_ENV_FILES", "1").lower() not in ["0", "false", ""]

default_settings_config: SettingsConfigDict

if use_default_env_files:
    default_settings_config = SettingsConfigDict(
        env_file=default_env_files,
        use_attribute_docstrings=True,
    )
else:
    default_settings_config = SettingsConfigDict(
        use_attribute_docstrings=True,
    )


class SharedCISettings(BaseSettings):
    """Shared settings for CI/CD pipelines."""

    model_config = default_settings_config

    tests_ongoing: bool = Field(default=False)
    """Indicates if any CI/CD pipeline is ongoing."""
